@@ -259,7 +259,7 @@ def ridge_regression(X, y, alpha, *, sample_weight=None, solver='auto',
         the estimates. Larger values specify stronger regularization.
         Alpha corresponds to ``1 / (2C)`` in other linear models such as
         :class:`~sklearn.linear_model.LogisticRegression` or
-        :class:`sklearn.svm.LinearSVC`. If an array is passed, penalties are
+        :class:`~sklearn.svm.LinearSVC`. If an array is passed, penalties are
         assumed to be specific to the targets. Hence they must correspond in
         number.
 
@@ -623,7 +623,7 @@ class Ridge(MultiOutputMixin, RegressorMixin, _BaseRidge):
         the estimates. Larger values specify stronger regularization.
         Alpha corresponds to ``1 / (2C)`` in other linear models such as
         :class:`~sklearn.linear_model.LogisticRegression` or
-        :class:`sklearn.svm.LinearSVC`. If an array is passed, penalties are
+        :class:`~sklearn.svm.LinearSVC`. If an array is passed, penalties are
         assumed to be specific to the targets. Hence they must correspond in
         number.
 
@@ -637,7 +637,7 @@ class Ridge(MultiOutputMixin, RegressorMixin, _BaseRidge):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     copy_X : bool, default=True
@@ -711,12 +711,12 @@ class Ridge(MultiOutputMixin, RegressorMixin, _BaseRidge):
 
         .. versionadded:: 0.17
 
-    See also
+    See Also
     --------
-    RidgeClassifier : Ridge classifier
-    RidgeCV : Ridge regression with built-in cross validation
-    :class:`sklearn.kernel_ridge.KernelRidge` : Kernel ridge regression
-        combines ridge regression with the kernel trick
+    RidgeClassifier : Ridge classifier.
+    RidgeCV : Ridge regression with built-in cross validation.
+    :class:`~sklearn.kernel_ridge.KernelRidge` : Kernel ridge regression
+        combines ridge regression with the kernel trick.
 
     Examples
     --------
@@ -779,7 +779,7 @@ class RidgeClassifier(LinearClassifierMixin, _BaseRidge):
         the estimates. Larger values specify stronger regularization.
         Alpha corresponds to ``1 / (2C)`` in other linear models such as
         :class:`~sklearn.linear_model.LogisticRegression` or
-        :class:`sklearn.svm.LinearSVC`.
+        :class:`~sklearn.svm.LinearSVC`.
 
     fit_intercept : bool, default=True
         Whether to calculate the intercept for this model. If set to false, no
@@ -791,7 +791,7 @@ class RidgeClassifier(LinearClassifierMixin, _BaseRidge):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     copy_X : bool, default=True
@@ -1074,9 +1074,7 @@ class _IdentityClassifier(LinearClassifierMixin):
 
 
 class _RidgeGCV(LinearModel):
-    """Ridge regression with built-in Generalized Cross-Validation.
-
-    It allows efficient Leave-One-Out cross-validation.
+    """Ridge regression with built-in Leave-one-out Cross-Validation.
 
     This class is not intended to be used directly. Use RidgeCV instead.
 
@@ -1120,7 +1118,7 @@ class _RidgeGCV(LinearModel):
                  fit_intercept=True, normalize=False,
                  scoring=None, copy_X=True,
                  gcv_mode=None, store_cv_values=False,
-                 is_clf=False):
+                 is_clf=False, alpha_per_target=False):
         self.alphas = np.asarray(alphas)
         self.fit_intercept = fit_intercept
         self.normalize = normalize
@@ -1129,6 +1127,7 @@ class _RidgeGCV(LinearModel):
         self.gcv_mode = gcv_mode
         self.store_cv_values = store_cv_values
         self.is_clf = is_clf
+        self.alpha_per_target = alpha_per_target
 
     @staticmethod
     def _decomp_diag(v_prime, Q):
@@ -1456,6 +1455,11 @@ class _RidgeGCV(LinearModel):
                                    dtype=[np.float64],
                                    multi_output=True, y_numeric=True)
 
+        # alpha_per_target cannot be used in classifier mode. All subclasses
+        # of _RidgeGCV that are classifiers keep alpha_per_target at its
+        # default value: False, so the condition below should never happen.
+        assert not (self.is_clf and self.alpha_per_target)
+
         if sample_weight is not None:
             sample_weight = _check_sample_weight(sample_weight, X,
                                                  dtype=X.dtype)
@@ -1496,19 +1500,23 @@ class _RidgeGCV(LinearModel):
         error = scorer is None
 
         n_y = 1 if len(y.shape) == 1 else y.shape[1]
+        n_alphas = 1 if np.ndim(self.alphas) == 0 else len(self.alphas)
 
         if self.store_cv_values:
             self.cv_values_ = np.empty(
-                (n_samples * n_y, len(self.alphas)), dtype=X.dtype)
+                (n_samples * n_y, n_alphas), dtype=X.dtype)
 
         best_coef, best_score, best_alpha = None, None, None
 
-        for i, alpha in enumerate(self.alphas):
+        for i, alpha in enumerate(np.atleast_1d(self.alphas)):
             G_inverse_diag, c = solve(
                 float(alpha), y, sqrt_sw, X_mean, *decomposition)
             if error:
                 squared_errors = (c / G_inverse_diag) ** 2
-                alpha_score = -squared_errors.mean()
+                if self.alpha_per_target:
+                    alpha_score = -squared_errors.mean(axis=0)
+                else:
+                    alpha_score = -squared_errors.mean()
                 if self.store_cv_values:
                     self.cv_values_[:, i] = squared_errors.ravel()
             else:
@@ -1520,15 +1528,40 @@ class _RidgeGCV(LinearModel):
                     identity_estimator = _IdentityClassifier(
                         classes=np.arange(n_y)
                     )
-                    predictions_, y_ = predictions, y.argmax(axis=1)
+                    alpha_score = scorer(identity_estimator,
+                                         predictions, y.argmax(axis=1))
                 else:
                     identity_estimator = _IdentityRegressor()
-                    predictions_, y_ = predictions.ravel(), y.ravel()
+                    if self.alpha_per_target:
+                        alpha_score = np.array([
+                            scorer(identity_estimator,
+                                   predictions[:, j], y[:, j])
+                            for j in range(n_y)
+                        ])
+                    else:
+                        alpha_score = scorer(identity_estimator,
+                                             predictions.ravel(), y.ravel())
 
-                alpha_score = scorer(identity_estimator, predictions_, y_)
-
-            if (best_score is None) or (alpha_score > best_score):
-                best_coef, best_score, best_alpha = c, alpha_score, alpha
+            # Keep track of the best model
+            if best_score is None:
+                # initialize
+                if self.alpha_per_target and n_y > 1:
+                    best_coef = c
+                    best_score = np.atleast_1d(alpha_score)
+                    best_alpha = np.full(n_y, alpha)
+                else:
+                    best_coef = c
+                    best_score = alpha_score
+                    best_alpha = alpha
+            else:
+                # update
+                if self.alpha_per_target and n_y > 1:
+                    to_update = alpha_score > best_score
+                    best_coef[:, to_update] = c[:, to_update]
+                    best_score[to_update] = alpha_score[to_update]
+                    best_alpha[to_update] = alpha
+                elif alpha_score > best_score:
+                    best_coef, best_score, best_alpha = c, alpha_score, alpha
 
         self.alpha_ = best_alpha
         self.best_score_ = best_score
@@ -1540,9 +1573,9 @@ class _RidgeGCV(LinearModel):
 
         if self.store_cv_values:
             if len(y.shape) == 1:
-                cv_values_shape = n_samples, len(self.alphas)
+                cv_values_shape = n_samples, n_alphas
             else:
-                cv_values_shape = n_samples, n_y, len(self.alphas)
+                cv_values_shape = n_samples, n_y, n_alphas
             self.cv_values_ = self.cv_values_.reshape(cv_values_shape)
 
         return self
@@ -1552,8 +1585,8 @@ class _BaseRidgeCV(LinearModel):
     @_deprecate_positional_args
     def __init__(self, alphas=(0.1, 1.0, 10.0), *,
                  fit_intercept=True, normalize=False, scoring=None,
-                 cv=None, gcv_mode=None,
-                 store_cv_values=False):
+                 cv=None, gcv_mode=None, store_cv_values=False,
+                 alpha_per_target=False):
         self.alphas = np.asarray(alphas)
         self.fit_intercept = fit_intercept
         self.normalize = normalize
@@ -1561,6 +1594,7 @@ class _BaseRidgeCV(LinearModel):
         self.cv = cv
         self.gcv_mode = gcv_mode
         self.store_cv_values = store_cv_values
+        self.alpha_per_target = alpha_per_target
 
     def fit(self, X, y, sample_weight=None):
         """Fit Ridge regression model with cv.
@@ -1585,8 +1619,8 @@ class _BaseRidgeCV(LinearModel):
         Notes
         -----
         When sample_weight is provided, the selected hyperparameter may depend
-        on whether we use generalized cross-validation (cv=None or cv='auto')
-        or another form of cross-validation, because only generalized
+        on whether we use leave-one-out cross-validation (cv=None or cv='auto')
+        or another form of cross-validation, because only leave-one-out
         cross-validation takes the sample weights into account when computing
         the validation score.
         """
@@ -1598,7 +1632,8 @@ class _BaseRidgeCV(LinearModel):
                                   scoring=self.scoring,
                                   gcv_mode=self.gcv_mode,
                                   store_cv_values=self.store_cv_values,
-                                  is_clf=is_classifier(self))
+                                  is_clf=is_classifier(self),
+                                  alpha_per_target=self.alpha_per_target)
             estimator.fit(X, y, sample_weight=sample_weight)
             self.alpha_ = estimator.alpha_
             self.best_score_ = estimator.best_score_
@@ -1606,7 +1641,10 @@ class _BaseRidgeCV(LinearModel):
                 self.cv_values_ = estimator.cv_values_
         else:
             if self.store_cv_values:
-                raise ValueError("cv!=None and store_cv_values=True "
+                raise ValueError("cv!=None and store_cv_values=True"
+                                 " are incompatible")
+            if self.alpha_per_target:
+                raise ValueError("cv!=None and alpha_per_target=True"
                                  " are incompatible")
             parameters = {'alpha': self.alphas}
             solver = 'sparse_cg' if sparse.issparse(X) else 'auto'
@@ -1632,7 +1670,7 @@ class RidgeCV(MultiOutputMixin, RegressorMixin, _BaseRidgeCV):
 
     See glossary entry for :term:`cross-validation estimator`.
 
-    By default, it performs Generalized Cross-Validation, which is a form of
+    By default, it performs Leave-One-Out Cross-Validation, which is a form of
     efficient Leave-One-Out cross-validation.
 
     Read more in the :ref:`User Guide <ridge_regression>`.
@@ -1646,8 +1684,8 @@ class RidgeCV(MultiOutputMixin, RegressorMixin, _BaseRidgeCV):
         the estimates. Larger values specify stronger regularization.
         Alpha corresponds to ``1 / (2C)`` in other linear models such as
         :class:`~sklearn.linear_model.LogisticRegression` or
-        :class:`sklearn.svm.LinearSVC`.
-        If using generalized cross-validation, alphas must be positive.
+        :class:`~sklearn.svm.LinearSVC`.
+        If using Leave-One-Out cross-validation, alphas must be positive.
 
     fit_intercept : bool, default=True
         Whether to calculate the intercept for this model. If set
@@ -1659,7 +1697,7 @@ class RidgeCV(MultiOutputMixin, RegressorMixin, _BaseRidgeCV):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     scoring : string, callable, default=None
@@ -1667,28 +1705,28 @@ class RidgeCV(MultiOutputMixin, RegressorMixin, _BaseRidgeCV):
         a scorer callable object / function with signature
         ``scorer(estimator, X, y)``.
         If None, the negative mean squared error if cv is 'auto' or None
-        (i.e. when using generalized cross-validation), and r2 score otherwise.
+        (i.e. when using leave-one-out cross-validation), and r2 score
+        otherwise.
 
     cv : int, cross-validation generator or an iterable, default=None
         Determines the cross-validation splitting strategy.
         Possible inputs for cv are:
 
         - None, to use the efficient Leave-One-Out cross-validation
-          (also known as Generalized Cross-Validation).
         - integer, to specify the number of folds.
         - :term:`CV splitter`,
         - An iterable yielding (train, test) splits as arrays of indices.
 
         For integer/None inputs, if ``y`` is binary or multiclass,
-        :class:`sklearn.model_selection.StratifiedKFold` is used, else,
-        :class:`sklearn.model_selection.KFold` is used.
+        :class:`~sklearn.model_selection.StratifiedKFold` is used, else,
+        :class:`~sklearn.model_selection.KFold` is used.
 
         Refer :ref:`User Guide <cross_validation>` for the various
         cross-validation strategies that can be used here.
 
     gcv_mode : {'auto', 'svd', eigen'}, default='auto'
         Flag indicating which strategy to use when performing
-        Generalized Cross-Validation. Options are::
+        Leave-One-Out Cross-Validation. Options are::
 
             'auto' : use 'svd' if n_samples > n_features, otherwise use 'eigen'
             'svd' : force use of singular value decomposition of X when X is
@@ -1702,16 +1740,25 @@ class RidgeCV(MultiOutputMixin, RegressorMixin, _BaseRidgeCV):
         Flag indicating if the cross-validation values corresponding to
         each alpha should be stored in the ``cv_values_`` attribute (see
         below). This flag is only compatible with ``cv=None`` (i.e. using
-        Generalized Cross-Validation).
+        Leave-One-Out Cross-Validation).
+
+    alpha_per_target : bool, default=False
+        Flag indicating whether to optimize the alpha value (picked from the
+        `alphas` parameter list) for each target separately (for multi-output
+        settings: multiple prediction targets). When set to `True`, after
+        fitting, the `alpha_` attribute will contain a value for each target.
+        When set to `False`, a single alpha is used for all targets.
+
+        .. versionadded:: 0.24
 
     Attributes
     ----------
     cv_values_ : ndarray of shape (n_samples, n_alphas) or \
         shape (n_samples, n_targets, n_alphas), optional
-        Cross-validation values for each alpha (only available if \
-        ``store_cv_values=True`` and ``cv=None``). After ``fit()`` has been \
-        called, this attribute will contain the mean squared errors \
-        (by default) or the values of the ``{loss,score}_func`` function \
+        Cross-validation values for each alpha (only available if
+        ``store_cv_values=True`` and ``cv=None``). After ``fit()`` has been
+        called, this attribute will contain the mean squared errors
+        (by default) or the values of the ``{loss,score}_func`` function
         (if provided in the constructor).
 
     coef_ : ndarray of shape (n_features) or (n_targets, n_features)
@@ -1721,11 +1768,15 @@ class RidgeCV(MultiOutputMixin, RegressorMixin, _BaseRidgeCV):
         Independent term in decision function. Set to 0.0 if
         ``fit_intercept = False``.
 
-    alpha_ : float
-        Estimated regularization parameter.
+    alpha_ : float or ndarray of shape (n_targets,)
+        Estimated regularization parameter, or, if ``alpha_per_target=True``,
+        the estimated regularization parameter for each target.
 
-    best_score_ : float
-        Score of base estimator with best alpha.
+    best_score_ : float or ndarray of shape (n_targets,)
+        Score of base estimator with best alpha, or, if
+        ``alpha_per_target=True``, a score for each target.
+
+        .. versionadded:: 0.23
 
     Examples
     --------
@@ -1736,11 +1787,11 @@ class RidgeCV(MultiOutputMixin, RegressorMixin, _BaseRidgeCV):
     >>> clf.score(X, y)
     0.5166...
 
-    See also
+    See Also
     --------
-    Ridge : Ridge regression
-    RidgeClassifier : Ridge classifier
-    RidgeClassifierCV : Ridge classifier with built-in cross validation
+    Ridge : Ridge regression.
+    RidgeClassifier : Ridge classifier.
+    RidgeClassifierCV : Ridge classifier with built-in cross validation.
     """
 
 
@@ -1749,9 +1800,8 @@ class RidgeClassifierCV(LinearClassifierMixin, _BaseRidgeCV):
 
     See glossary entry for :term:`cross-validation estimator`.
 
-    By default, it performs Generalized Cross-Validation, which is a form of
-    efficient Leave-One-Out cross-validation. Currently, only the n_features >
-    n_samples case is handled efficiently.
+    By default, it performs Leave-One-Out Cross-Validation. Currently,
+    only the n_features > n_samples case is handled efficiently.
 
     Read more in the :ref:`User Guide <ridge_regression>`.
 
@@ -1764,7 +1814,7 @@ class RidgeClassifierCV(LinearClassifierMixin, _BaseRidgeCV):
         the estimates. Larger values specify stronger regularization.
         Alpha corresponds to ``1 / (2C)`` in other linear models such as
         :class:`~sklearn.linear_model.LogisticRegression` or
-        :class:`sklearn.svm.LinearSVC`.
+        :class:`~sklearn.svm.LinearSVC`.
 
     fit_intercept : bool, default=True
         Whether to calculate the intercept for this model. If set
@@ -1776,7 +1826,7 @@ class RidgeClassifierCV(LinearClassifierMixin, _BaseRidgeCV):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     scoring : string, callable, default=None
@@ -1808,7 +1858,7 @@ class RidgeClassifierCV(LinearClassifierMixin, _BaseRidgeCV):
         Flag indicating if the cross-validation values corresponding to
         each alpha should be stored in the ``cv_values_`` attribute (see
         below). This flag is only compatible with ``cv=None`` (i.e. using
-        Generalized Cross-Validation).
+        Leave-One-Out Cross-Validation).
 
     Attributes
     ----------
@@ -1834,6 +1884,8 @@ class RidgeClassifierCV(LinearClassifierMixin, _BaseRidgeCV):
     best_score_ : float
         Score of base estimator with best alpha.
 
+        .. versionadded:: 0.23
+
     classes_ : ndarray of shape (n_classes,)
         The classes labels.
 
@@ -1846,11 +1898,11 @@ class RidgeClassifierCV(LinearClassifierMixin, _BaseRidgeCV):
     >>> clf.score(X, y)
     0.9630...
 
-    See also
+    See Also
     --------
-    Ridge : Ridge regression
-    RidgeClassifier : Ridge classifier
-    RidgeCV : Ridge regression with built-in cross validation
+    Ridge : Ridge regression.
+    RidgeClassifier : Ridge classifier.
+    RidgeCV : Ridge regression with built-in cross validation.
 
     Notes
     -----
@@ -1909,3 +1961,11 @@ class RidgeClassifierCV(LinearClassifierMixin, _BaseRidgeCV):
     @property
     def classes_(self):
         return self._label_binarizer.classes_
+
+    def _more_tags(self):
+        return {
+            '_xfail_checks': {
+                'check_sample_weights_invariance':
+                'zero sample_weight is not equivalent to removing samples',
+            }
+        }
